/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 24/11/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.constants.Constants;
import eu.europa.ec.re3gistrycommon.model.Itemparent;
import eu.europa.ec.re3gistrycommon.model.Item;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class ItemparentMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public ItemparentMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the itemparent with the specified uuid.
     *
     * @param uuid the unique id of the itemparent
     * @return the itemparent with the specified uuid
     * @throws Exception
     */
    public Itemparent getItemparentByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemparent.findByUuid");
        q.setParameter("uuid", uuid);
        return (Itemparent) q.getSingleResult();
    }

    public List<Itemparent> findByItem(Item item) throws Exception {
        //Checking parameters
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemparent.findByItem");
        q.setParameter("item", item);
        return (List<Itemparent>) q.getResultList();
    }

    public List<Itemparent> findByParent(Item parent) throws Exception {
        //Checking parameters
        if (parent == null) {
            throw new Exception("The parent item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemparent.findByParent");
        q.setParameter("parent", parent);
        return (List<Itemparent>) q.getResultList();
    }
    
    public List<Itemparent> findByParentAndItemClass(Item parent) throws Exception {
        //Checking parameters
        if (parent == null) {
            throw new Exception("The parent item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemparent.findByParentAndItemClass");
        q.setParameter("parent", parent);
        q.setParameter("itemclass", parent.getItemclass());
        return (List<Itemparent>) q.getResultList();
    }

    /**
     * Insert the specified itempredecessors in the database. Returns true if
     * the operation succeed.
     *
     * @param ip the itemparent object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertItemparent(Itemparent ip) throws Exception {
        //Checking parameters
        if (ip == null) {
            throw new Exception("The Itemparent passed by parameter is null.");
        }

        if (ip.getDatecreation() == null) {
            ip.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ip);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(ip);

        return true;
    }

    /**
     * Update the itemparent specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param ip the updated itemparent (this method update the itemparent on
     * the db with the itemparent passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateItemparent(Itemparent ip) throws Exception {
        //Checking parameters
        if (ip == null) {
            throw new Exception("The Itemparent passed by parameter is null.");
        }

        if (ip.getDatelastupdate() == null) {
            ip.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ip, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(ip);

        return true;
    }

    /**
     * Delete from the database the itemparent specified by parameter. Returns
     * true if the operation succeed.
     *
     * @param ip the itemparent do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteItemparent(Itemparent ip) throws Exception {
        //Checking parameters
        if (ip == null) {
            throw new Exception("The Itemparent passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(ip);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Itemparent ip) throws Exception {
        if (ip.getUuid() == null || ip.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ip.getItem() == null) {
            throw new Exception("Item field not available.");
        }

        if (ip.getParent() == null) {
            throw new Exception("Parent field not available.");
        }

        if (ip.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Itemparent ip, boolean isUpdate) throws Exception {
        if (ip.getUuid() == null || ip.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ip.getItem() == null) {
            throw new Exception("Item field not available.");
        }

        if (ip.getParent() == null) {
            throw new Exception("Parent field not available.");
        }

        if (ip.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }
        if (isUpdate && ip.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }

    public Itemparent getLatestItemparent(Item item) throws Exception {
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNativeQuery(Constants.SQL_LARESTVERSION_OF_PARENT_BY_ITEM, Itemparent.class);
        q.setParameter(1, item.getUuid());

        Itemparent itemparent = null;
        try {
            itemparent = (Itemparent) q.getSingleResult();
        } catch (Exception e) {
            throw new Exception("Error while executing query: " + e.getMessage());
        }

        return itemparent;
    }

    public List<Itemparent> getLatestItemparentItemByParent(Item parent) throws Exception {
        if (parent == null) {
            throw new Exception("The parent passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNativeQuery(Constants.SQL_LARESTVERSION_OF_ITEM_BY_PARENT, Itemparent.class);
        q.setParameter(1, parent.getUuid());

        List<Itemparent> itemparent = null;
        try {
            itemparent = (List<Itemparent>) q.getResultList();
        } catch (Exception e) {
            throw new Exception("Error while executing query: " + e.getMessage());
        }

        return itemparent;
    }

    public void cleanOldChildrenRelations(Item i) throws Exception{
        if(i == null){
            throw new Exception("The item passed by parameter is null.");
        }
        
        try {
            Query query = em.createNativeQuery(Constants.SQL_CLEAN_CHILDREN_REL);
            query.setParameter(1,i.getUuid());
            query.executeUpdate();
        } catch (Exception e) {
            throw new Exception("Error while executing query: " + e.getMessage());
        }
    }
    public void cleanOldParentRelations(Item i) throws Exception{
        if(i == null){
            throw new Exception("The item passed by parameter is null.");
        }
        
        try {
            Query query = em.createNativeQuery(Constants.SQL_CLEAN_PARENT_REL);
            query.setParameter(1,i.getUuid());
            query.executeUpdate();
        } catch (Exception e) {
            throw new Exception("Error while executing query: " + e.getMessage());
        }
    }
}
