/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 24/11/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.constants.Constants;
import eu.europa.ec.re3gistrycommon.model.Users;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.util.Date;
import java.util.Properties;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.servlet.http.HttpServletRequest;
import org.apache.commons.io.IOUtils;

public class UserMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public UserMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns all staticizer
     *
     * @param email
     * @return all the staticizer
     * @throws Exception
     */
    public Users getUserByEmail(String email) throws Exception {
        Query q = this.em.createNamedQuery("Users.findByEmail");
        q.setParameter("email", email);
        return (Users) q.getSingleResult();
    }

    public Users checkUserByEmail(String email) {
        Users u;

        try {
            u = getUserByEmail(email);
        } catch (Exception e) {
            try {
                u = new Users();
                u.setUuid(Users.createUuid(email));
                u.setEmail(email);
                u.setEnable(true);

                insertUser(u);

            } catch (Exception ex) {
                u = null;
            }
        }

        return u;
    }

    /**
     * Insert the specified items in the database. Returns true if the operation
     * succeed.
     *
     * @param u the user object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertUser(Users u) throws Exception {
        //Checking parameters
        if (u == null) {
            throw new Exception("The User passed by parameter is null.");
        }

        if (u.getDatecreation() == null) {
            u.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(u);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        em.getTransaction().begin();

        //Saving the object
        this.em.persist(u);

        em.getTransaction().commit();

        return true;
    }

    public String getLogoutLink(HttpServletRequest request) throws Exception {
        String url = request.getHeader("referer");
        String contextPath = request.getContextPath();
        String homeUrl = "";
        if (contextPath != null) {
            int indexOfContextPath = url.lastIndexOf(contextPath);
            int contextPathLength = contextPath.length();
            if (indexOfContextPath > 0 && contextPathLength > 0) {
                int homeUrlLength = indexOfContextPath + contextPathLength;
                homeUrl = url.substring(0, homeUrlLength) + "/";
            }
        }
        String logoutLink = getProperties(Constants.ECAS_FILE).getProperty("eu.cec.digit.ecas.client.filter.ecasBaseUrl") + "/cas/logout?url=" + homeUrl;
        request.getSession(false).invalidate();

        return logoutLink;
    }

    /**
     * Get a resource bundle
     *
     * @param path Name of the file
     * @return
     * @throws URISyntaxException
     * @throws FileNotFoundException
     * @throws IOException
     */
    private static Properties getProperties(String path) throws URISyntaxException, FileNotFoundException, IOException, Exception {
        Properties props = new Properties(System.getProperties());
        File propertiesFile = new File(UserMgr.class.getClassLoader().getResource(path).toURI());
        if (!propertiesFile.exists()) {
            throw new FileNotFoundException("Properties file not found: " + propertiesFile.getAbsolutePath());
        }
        InputStream inStream = null;
        try {
            inStream = new FileInputStream(propertiesFile);
            props.load(inStream);
            inStream.close();
        } catch (Exception e) {
            if (inStream != null) {
                inStream.close();
            }
            throw e;
        }
        return props;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Users u) throws Exception {
        if (u.getUuid() == null || u.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }
        if (u.getEmail() == null || u.getEmail().trim().equals("")) {
            throw new Exception("Email field not available.");
        }

        return true;
    }
}
