/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 24/11/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrydata.managers.actions;

import eu.europa.ec.re3gistrycommon.managers.ChangelogMgr;
import eu.europa.ec.re3gistrycommon.managers.CustomattributeMgr;
import eu.europa.ec.re3gistrycommon.managers.CustomattributevalueMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemclassMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemcollectionMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemparentMgr;
import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.managers.StatusMgr;
import eu.europa.ec.re3gistrycommon.model.Changelog;
import eu.europa.ec.re3gistrycommon.model.Customattribute;
import eu.europa.ec.re3gistrycommon.model.Customattributevalue;
import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Itemparent;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Status;
import eu.europa.ec.re3gistrydata.beans.Addition;
import eu.europa.ec.re3gistrydata.beans.CustomattributeDescriptor;
import eu.europa.ec.re3gistrydata.constants.Constants;
import eu.europa.ec.re3gistrydata.exceptions.ErrorReport;
import eu.europa.ec.re3gistrydata.exceptions.WarningReport;
import eu.europa.ec.re3gistrydata.managers.DataMgr;
import eu.europa.ec.re3gistrydata.managers.PropertyMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;

public class AdditionMgr {

    //static variables
    private static final Logger log = Logger.getLogger(Constants.MODULE_NAME);

    private final Properties properties;
    private final EntityManager em;
    private final ReportMgr reportMgr;
    private final Dataprocedure dataprocedure;

    private final String latestVersion;

    //Hashmaps
    private final LinkedHashMap<String, Addition> additions;

    //Element to be inserted after the parent/collectio 
    private final List<Addition> waitForParent;
    private final List<Addition> waitForCollection;

    //Performance store (in order to avoid continuous access to the db)
    private final LinkedHashMap<String, Customattribute> customattributeLoaded = new LinkedHashMap<String, Customattribute>();
    private final LinkedHashMap<String, Customattributevalue> customattributevalueLoaded = new LinkedHashMap<String, Customattributevalue>();
    private final LinkedHashMap<String, Languagecode> languagecodeLoaded = new LinkedHashMap<String, Languagecode>();
    private final LinkedHashMap<String, Item> itemLoaded = new LinkedHashMap<String, Item>();

    //Localization
    private final ResourceBundle localizationres;

    /**
     * Class initializer
     *
     * @param em the EntityManager instance
     * @param reportMgr the ReportMgr
     * @param dataMgr the StoreMgr
     * @param dataprocedure
     */
    public AdditionMgr(EntityManager em, ReportMgr reportMgr, DataMgr dataMgr, Dataprocedure dataprocedure) {
        this.em = em;
        this.reportMgr = reportMgr;
        this.dataprocedure = dataprocedure;

        //Loading module properties
        this.properties = PropertyMgr.getProperties();

        //Initializing hashmap
        this.additions = dataMgr.getAdditions();

        this.waitForParent = new ArrayList<Addition>();
        this.waitForCollection = new ArrayList<Addition>();

        this.latestVersion = properties.getProperty(Constants.KEY_LATEST_VERSION);

        HttpSession s = dataMgr.getRequest().getSession();
        String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
        if (currentLanguage == null) {
            currentLanguage = dataMgr.getRequest().getLocale().toString();
        }
        this.localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));

    }

    /**
     * Store the related additions in the database
     *
     * @throws Exception
     */
    public void storeAdditions() {
        log.info("# Starting to store additions.");

        Itemclass itemclass = null;
        Status status = null;

        int additionCounter = 0;

        //Starting the store of each addition
        for (String key : this.additions.keySet()) {
            try {
                Addition addition = this.additions.get(key);

                /*Getting itemclass and status out of the storeAddition method
                 for performance reason.*/
                //Getting itemclass
                if (addition.getItemClass() == null || addition.getItemClass().length() <= 0) {
                    throw new Exception(this.localizationres.getString("error.additionmgr.stad.noitmc"));
                }
                if (itemclass == null || (!addition.getItemClass().equals(itemclass.getUriname()))) {
                    itemclass = this.getItemclass(addition.getItemClass());
                }

                //Storing the current addition
                Addition a = this.additions.get(key);
                if (a.isExternalItem()) {
                    if (addition.getStatus()!=null && addition.getStatus().length()>0) {
                        status = this.getStatus(addition.getStatus());
                    }else{
                        status = null;
                    }
                    
                    this.storeAdditionExternal(a, itemclass, status);
                } else {
                    //Getting status
                    if (addition.getStatus() == null || addition.getStatus().length() <= 0) {
                        throw new Exception(this.localizationres.getString("error.additionmgr.stad.nost"));
                    }
                    if (status == null || (!addition.getStatus().equals(status.getUriname()))) {
                        status = this.getStatus(addition.getStatus());
                    }
                    this.storeAddition(a, itemclass, status);
                }
                additionCounter++;

            } catch (Exception e) {
                //If an error occurs during the DB operation for the current 
                //additions, skips the current addition and saves a 
                //warning message
                reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_STOREADDITION, "This item is skipped: " + key + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.error("@@ This element is skipped: " + key + ". Detail: " + e.getMessage());
            }
        }

        //Check if the system store some additions succesfully
        if (additionCounter > 0) {
            //Fix the parent/collection (if some child /collection item is inserted
            //before the related parent)
            this.fixParentCollection();

            log.info("# Additions stored.");
        } else if (this.additions.size() == 0) {
            log.info("# There are no addition to store.");
        } else {
            //If no addition were stored, it store an error in the reports
            reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_STOREADDITION, "The store procedure skip all the elements; check the related messages.", Constants.REPORT_TYPE_ERROR, -1);
            log.error("@@ The store procedure skip all the elements; check the related messages.");
            log.info("# No additions stored.");
        }
    }

    private void storeAddition(Addition addition, Itemclass itemClass, Status status) throws Exception {

        //Initializing managers
        ItemMgr itemMgr = new ItemMgr(em);
        LocalizationMgr localizationMgr = new LocalizationMgr(this.em);

        //Creating the item UUID;
        String itemUuid = Item.createUuid(addition.getLocalId(), this.latestVersion, addition.getCollectionLocalId(), addition.getItemClass());

        //Starting operation on the item
        Item item;
        if (itemLoaded.containsKey(itemUuid)) {
            item = itemLoaded.get(itemUuid);
        } else {
            try {
                //Checing if the item is already in the DB (in this case try to  
                //insert the localization);
                item = itemMgr.getItemByUuid(itemUuid);
                itemLoaded.put(itemUuid, item);
            } catch (NoResultException ex) {

                //Checking itemclass
                if (itemClass == null) {
                    String tmp = this.localizationres.getString("error.additionmgr.stad.itmcn").replace("{0}", addition.getItemClass()).replace("{1}", addition.getLocalId());
                    throw new Exception(tmp);
                }
                if (!itemClass.getUriname().equals(addition.getItemClass())) {
                    String tmp = this.localizationres.getString("error.additionmgr.stad.itmcmsm").replace("{0}", addition.getItemClass()).replace("{1}", itemClass.getUriname());
                    throw new Exception(tmp);
                }

                //Checking status
                if (status == null) {
                    String tmp = this.localizationres.getString("error.additionmgr.stad.stn").replace("{0}", addition.getItemClass()).replace("{1}", addition.getStatus());
                    throw new Exception(tmp);
                }
                if (!itemClass.getUriname().equals(addition.getItemClass())) {
                    String tmp = this.localizationres.getString("error.additionmgr.stad.itmcmsm").replace("{0}", addition.getItemClass()).replace("{1}", itemClass.getUriname());
                    throw new Exception(tmp);
                }

                //Creating the new item's object
                item = new Item();

                //Setting properties
                item.setUuid(itemUuid);
                item.setUriname(addition.getLocalId());
                item.setItemclass(itemClass);
                item.setStatus(status);
                item.setVersionnumber(Integer.parseInt(this.latestVersion));
                item.setDatecreation(new Date());
                item.setDatetimeaddition(new Date());

                //Saving the new item
                itemMgr.insertItem(item);

                //Getting parent item (if available)
                Item parent = null;
                if (addition.getParentLocalId() != null && addition.getParentLocalId().length() > 0) {

                    //Checking if there is multiple parents
                    String[] parents = addition.getParentLocalId().split(Constants.MULTIPLEPARENT_SEPARATOR);
                    for (String parentItem : parents) {

                        String parentItemUuid = Item.createUuid(parentItem, this.latestVersion, addition.getCollectionLocalId(), addition.getItemClass());
                        try {
                            parent = itemMgr.getItemByUuid(parentItemUuid);

                            //Set parent
                            if (parent != null) {
                                this.createNewParentRelation(item, parent);
                                this.storeAdditionChangelog(parent, addition);
                            }

                        } catch (NoResultException e) {
                            //If the parent is not available, store the current
                            //element in a list of item to be processed afrer the 
                            //insert of all the element in the current additions
                            Addition tmp = new Addition();
                            tmp.setLocalId(addition.getLocalId());
                            tmp.setCollectionLocalId(addition.getCollectionLocalId());
                            tmp.setParentLocalId(parentItem);
                            tmp.setItemClass(addition.getItemClass());
                            this.waitForParent.add(tmp);
                        }
                    }
                }

                //Getting collection item (if available)
                Item collection = null;
                if (addition.getCollectionLocalId() != null && addition.getCollectionLocalId().length() > 0) {

                    String collectionItemUuid = Item.createUuid(addition.getCollectionLocalId(), this.latestVersion, null, itemClass.getParent().getUriname());
                    try {
                        collection = itemMgr.getItemByUuid(collectionItemUuid);

                    } catch (NoResultException e) {
                        //If the collection parent is not available, store the 
                        //current element in a list of item to be processed 
                        //afrer the insert of all the element in the current additions
                        this.waitForCollection.add(addition);
                    }
                }

                //Set collection
                if (collection != null) {
                    this.createNewCollectionRelation(item, collection);
                    this.storeAdditionChangelog(collection, addition);
                }

                log.debug("# Storing new item: " + item.getUriname() + " - " + item.getItemclass().getUriname());
            }
        }

        if (item != null) {
            //Getting current language code
            Languagecode languagecode = null;
            if (languagecodeLoaded.containsKey(addition.getLanguage())) {
                languagecode = languagecodeLoaded.get(addition.getLanguage());
            } else {
                languagecode = this.getLanguagecode(addition.getLanguage());
                languagecodeLoaded.put(addition.getLanguage(), languagecode);
            }

            try {
                //Storing localization
                this.storeLocalization(addition, item, languagecode, localizationMgr);

                //Storing custom attributes
                this.storeCustomAttributeValue(addition, item, languagecode, localizationMgr);

            } catch (Exception e) {
                //If an error occurs during the DB operation for the current 
                //localization/custom attribute value, skips the current  
                //addition and saves a warning message
                reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_STOREADDITION, "The localization for this item is skipped: " + item.getUriname() + " - " + languagecode.getIsocode() + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.error("@@ The localization for this item is skipped: " + item.getUriname() + " - " + languagecode.getIsocode() + ". Detail: " + e.getMessage());

            }

            this.storeAdditionChangelog(item, addition);
        }
    }

    private void storeAdditionExternal(Addition addition, Itemclass itemClass, Status status) throws Exception {

        //Initializing managers
        ItemMgr itemMgr = new ItemMgr(em);
        LocalizationMgr localizationMgr = new LocalizationMgr(this.em);

        //Creating the item UUID;
        String itemUuid = Item.createUuid(addition.getLocalId(), this.latestVersion, addition.getCollectionLocalId(), addition.getItemClass());

        //Starting operation on the item
        Item item;
        if (itemLoaded.containsKey(itemUuid)) {
            item = itemLoaded.get(itemUuid);
        } else {
            try {
                //Checing if the item is already in the DB (in this case try to  
                //insert the localization);
                item = itemMgr.getItemByUuid(itemUuid);
                itemLoaded.put(itemUuid, item);
            } catch (NoResultException ex) {

                //Checking itemclass
                if (itemClass == null) {
                    String tmp = this.localizationres.getString("error.additionmgr.stad.itmcn").replace("{0}", addition.getItemClass()).replace("{1}", addition.getLocalId());
                    throw new Exception(tmp);
                }
                if (!itemClass.getUriname().equals(addition.getItemClass())) {
                    String tmp = this.localizationres.getString("error.additionmgr.stad.itmcmsm").replace("{0}", addition.getItemClass()).replace("{1}", itemClass.getUriname());
                    throw new Exception(tmp);
                }

                //Checking status
                /*if (status == null) {
                    String tmp = this.localizationres.getString("error.additionmgr.stad.stn").replace("{0}", addition.getItemClass()).replace("{1}", addition.getStatus());
                    throw new Exception(tmp);
                }*/
                if (!itemClass.getUriname().equals(addition.getItemClass())) {
                    String tmp = this.localizationres.getString("error.additionmgr.stad.itmcmsm").replace("{0}", addition.getItemClass()).replace("{1}", itemClass.getUriname());
                    throw new Exception(tmp);
                }

                //Creating the new item's object
                item = new Item();

                //Setting properties
                item.setUuid(itemUuid);
                item.setUriname(addition.getLocalId());
                item.setItemclass(itemClass);
                if(status!=null){
                    item.setStatus(status);
                }
                item.setVersionnumber(Integer.parseInt(this.latestVersion));
                item.setDatecreation(new Date());
                item.setDatetimeaddition(new Date());

                //Saving the new item
                itemMgr.insertItem(item);

                //Getting parent item (if available)
                Item parent = null;
                if (addition.getParentLocalId() != null && addition.getParentLocalId().length() > 0) {

                    //Checking if there is multiple parents
                    String[] parents = addition.getParentLocalId().split(Constants.MULTIPLEPARENT_SEPARATOR);
                    for (String parentItem : parents) {

                        String parentItemUuid = Item.createUuid(parentItem, this.latestVersion, addition.getCollectionLocalId(), addition.getItemClass());
                        try {
                            parent = itemMgr.getItemByUuid(parentItemUuid);

                            //Set parent
                            if (parent != null) {
                                this.createNewParentRelation(item, parent);
                                this.storeAdditionChangelog(parent, addition);
                            }

                        } catch (NoResultException e) {
                            //If the parent is not available, store the current
                            //element in a list of item to be processed afrer the 
                            //insert of all the element in the current additions
                            Addition tmp = new Addition();
                            tmp.setLocalId(addition.getLocalId());
                            tmp.setCollectionLocalId(addition.getCollectionLocalId());
                            tmp.setParentLocalId(parentItem);
                            tmp.setItemClass(addition.getItemClass());
                            this.waitForParent.add(tmp);
                        }
                    }
                }

                //Getting collection item (if available)
                Item collection = null;
                if (addition.getCollectionLocalId() != null && addition.getCollectionLocalId().length() > 0) {

                    String collectionItemUuid = Item.createUuid(addition.getCollectionLocalId(), this.latestVersion, null, itemClass.getParent().getUriname());
                    try {
                        collection = itemMgr.getItemByUuid(collectionItemUuid);

                    } catch (NoResultException e) {
                        //If the collection parent is not available, store the 
                        //current element in a list of item to be processed 
                        //afrer the insert of all the element in the current additions
                        this.waitForCollection.add(addition);
                    }
                }

                //Set collection
                if (collection != null) {
                    this.createNewCollectionRelation(item, collection);
                    this.storeAdditionChangelog(collection, addition);
                }

                log.debug("# Storing new item: " + item.getUriname() + " - " + item.getItemclass().getUriname());
            }
        }

        if (item != null && addition.getLanguage()!=null && addition.getLabel()!=null && addition.getLabel().length()>0) {
            //Getting current language code
            Languagecode languagecode = null;
            if (languagecodeLoaded.containsKey(addition.getLanguage())) {
                languagecode = languagecodeLoaded.get(addition.getLanguage());
            } else {
                languagecode = this.getLanguagecode(addition.getLanguage());
                languagecodeLoaded.put(addition.getLanguage(), languagecode);
            }

            try {
                //Storing localization
                this.storeLocalization(addition, item, languagecode, localizationMgr);

                //Storing custom attributes
                this.storeCustomAttributeValue(addition, item, languagecode, localizationMgr);

            } catch (Exception e) {
                //If an error occurs during the DB operation for the current 
                //localization/custom attribute value, skips the current  
                //addition and saves a warning message
                reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_STOREADDITION, "The localization for this item is skipped: " + item.getUriname() + " - " + languagecode.getIsocode() + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.error("@@ The localization for this item is skipped: " + item.getUriname() + " - " + languagecode.getIsocode() + ". Detail: " + e.getMessage());

            }

            this.storeAdditionChangelog(item, addition);
        }
    }

    private void fixParentCollection() {
        ItemMgr itemMgr = new ItemMgr(em);

        log.info("# Starting parent collection fix");

        //Updating the element in the "wait for parent list"
        if (this.waitForParent.size() >= 0) {
            for (Addition addition : this.waitForParent) {

                try {
                    //Getting the item to be updated
                    Item item;
                    String itemUuid = Item.createUuid(addition.getLocalId(), this.latestVersion, addition.getCollectionLocalId(), addition.getItemClass());
                    if (itemLoaded.containsKey(itemUuid)) {
                        item = itemLoaded.get(itemUuid);
                    } else {
                        item = itemMgr.getItemByUuid(itemUuid);
                    }

                    //Getting the item parent
                    String itemParentUuid = Item.createUuid(addition.getParentLocalId(), this.latestVersion, addition.getCollectionLocalId(), addition.getItemClass());
                    Item parent;
                    if (itemLoaded.containsKey(itemUuid)) {
                        parent = itemLoaded.get(itemParentUuid);
                    } else {
                        parent = itemMgr.getItemByUuid(itemParentUuid);
                    }

                    //Updating the itemparent relation
                    this.createNewParentRelation(item, parent);

                } catch (NoResultException e) {
                    //If an error occurs during the DB operation for the current 
                    //item/parent attribute value, skips the current  
                    //update and saves a warning message
                    reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_FIXPARENTCOLLECTION, "Addition manager, fix parent. Cannot load the item: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                    log.error("@@ Addition manager, fix parent. Cannot load the item: " + e.getMessage());
                } catch (Exception e) {
                    //If an error occurs during the DB operation for the current 
                    //item/parent attribute value, skips the current  
                    //update and saves a warning message
                    reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_FIXPARENTCOLLECTION, "Addition manager, fix parent. Cannot set the parent: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                    log.error("@@ Addition manager, fix parent. Cannot set the parent: " + e.getMessage());
                }
            }
        }

        if (this.waitForCollection.size() >= 0) {
            for (Addition addition : this.waitForCollection) {

                try {
                    //Getting the item to be updated
                    Item item;
                    String itemUuid = Item.createUuid(addition.getLocalId(), this.latestVersion, null, addition.getItemClass());
                    if (itemLoaded.containsKey(itemUuid)) {
                        item = itemLoaded.get(itemUuid);
                    } else {
                        item = itemMgr.getItemByUuid(itemUuid);
                    }

                    //Getting the collection parent
                    Item collection;
                    String itemCollectionUuid = Item.createUuid(addition.getCollectionLocalId(), this.latestVersion, null, item.getItemclass().getParent().getUriname());
                    if (itemLoaded.containsKey(itemCollectionUuid)) {
                        collection = itemLoaded.get(itemCollectionUuid);
                    } else {
                        collection = itemMgr.getItemByUuid(itemCollectionUuid);
                    }

                    //Updating the itemcollection relation
                    this.createNewCollectionRelation(item, collection);

                } catch (NoResultException e) {
                    //If an error occurs during the DB operation for the current 
                    //item/collection attribute value, skips the current  
                    //update and saves a warning message
                    reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_FIXPARENTCOLLECTION, "Addition manager, fix collection. Cannot load the item: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                    log.error("@@ Addition manager, fix collection. Cannot load the item: " + e.getMessage());
                } catch (Exception e) {
                    //If an error occurs during the DB operation for the current 
                    //item/collection attribute value, skips the current  
                    //update and saves a warning message
                    reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_FIXPARENTCOLLECTION, "Addition manager, fix collection. Cannot set the parent: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                    log.error("@@ Addition manager, fix collection. Cannot set the parent: " + e.getMessage());
                }
            }
        }

        log.info("# Parent collection fix completed");
    }

    private void storeLocalization(Addition addition, Item item, Languagecode languagecode, LocalizationMgr localizationMgr) throws Exception {
        Localization localization = null;
        String localizationUuid = Localization.createUuid(item.getUuid(), addition.getLanguage());

        //Checking if the element is already in the db        
        try {
            localization = localizationMgr.getLocalizationByUuid(localizationUuid);
        } catch (Exception e) {
        }

        if (localization != null) {
            //If the localization already exist, skip the current insert
            throw new Exception(this.localizationres.getString("error.additionmgr.stloc.exist"));
        }

        try {
            //Creating the new localization element
            localization = new Localization();
            localization.setUuid(localizationUuid);
            localization.setItem(item);
            localization.setLanguage(languagecode);
            localization.setLabel(addition.getLabel());
            localization.setDefinition(addition.getDefinition());
            localization.setDescription(addition.getDescription());
            localization.setDatecreation(new Date());

            //Storing the new localization
            localizationMgr.insertLocalization(localization);
            log.debug("# Storing localization: " + localization.getLabel() + " - " + item.getUriname() + " - " + languagecode.getIsocode() + " - " + item.getItemclass().getUriname());

        } catch (Exception e) {
            throw new Exception(this.localizationres.getString("error.additionmgr.stloc.err"));
        }
    }

    private void storeCustomAttributeValue(Addition addition, Item item, Languagecode languagecode, LocalizationMgr localizationMgr) throws Exception {
        CustomattributeMgr customAttributeMgr = new CustomattributeMgr(this.em);
        CustomattributevalueMgr customAttributeValueMgr = new CustomattributevalueMgr(this.em);

        List<CustomattributeDescriptor> customAttributeDescriptorsList = addition.getCustomAttributeDescriptors();
        for (CustomattributeDescriptor customAttributeDescriptor : customAttributeDescriptorsList) {

            if (customAttributeDescriptor.getValue() != null && customAttributeDescriptor.getValue().length() > 0) {

                //Getting the custom attribute descriptor
                Customattribute customAttribute;
                if (customattributeLoaded.containsKey(customAttributeDescriptor.getName())) {
                    customAttribute = customattributeLoaded.get(customAttributeDescriptor.getName());

                } else {
                    try {
                        //Getting the custom attribute descriptod (previously stored)
                        customAttribute = customAttributeMgr.getCustomattributeByName(customAttributeDescriptor.getName());
                        customattributeLoaded.put(customAttributeDescriptor.getName(), customAttribute);

                    } catch (NoResultException e) {
                        throw new Exception(this.localizationres.getString("error.additionmgr.setcav.nor").replace("{0}", customAttributeDescriptor.getName()));
                    } catch (Exception e) {
                        throw new Exception(this.localizationres.getString("error.additionmgr.setcav.err").replace("{0}", e.getMessage()));
                    }
                }

                //Checking if the custom attribute value descriptor is already in 
                //the db. If so, store the localized value
                Customattributevalue customAttributeValue;
                String itmcavUuid = Customattributevalue.createUuid(customAttributeDescriptor.getName(), item.getUuid(), item.getItemclass().getUriname());
                if (customattributevalueLoaded.containsKey(itmcavUuid)) {
                    customAttributeValue = customattributevalueLoaded.get(itmcavUuid);
                } else {
                    try {
                        customAttributeValue = customAttributeValueMgr.getCustomattributevalueByUuid(itmcavUuid);
                        customattributevalueLoaded.put(itmcavUuid, customAttributeValue);

                    } catch (NoResultException e) {
                        //If the customattribute value is not in the db, store it
                        customAttributeValue = new Customattributevalue();
                        customAttributeValue.setUuid(itmcavUuid);
                        customAttributeValue.setCustomattribute(customAttribute);
                        customAttributeValue.setItem(item);
                        customAttributeValue.setDatecreation(new Date());

                        customAttributeValueMgr.insertCustomattributevalue(customAttributeValue);
                        customattributevalueLoaded.put(itmcavUuid, customAttributeValue);

                    } catch (Exception e) {
                        throw new Exception(this.localizationres.getString("error.additionmgr.setcav.errs").replace("{0}", e.getMessage()));
                    }
                }

                //creating the localization element for the custom attribute value
                Localization localization = new Localization();
                String localizationUuid = Localization.createUuid(itmcavUuid + item.getUuid(), languagecode.getIsocode());
                localization.setUuid(localizationUuid);
                localization.setLanguage(languagecode);
                localization.setCustomattributevalue(customAttributeValue);
                localization.setLabel(customAttributeDescriptor.getValue());
                localization.setDatecreation(new Date());

                localizationMgr.insertLocalization(localization);
                log.debug("# Storing custom attribute value: " + customAttributeDescriptor.getName() + ": " + customAttributeDescriptor.getValue() + " - " + languagecode.getIsocode());

            }

        }
    }

    private Languagecode getLanguagecode(String isocode) throws Exception {
        //Getting current languace code
        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
        Languagecode languacecode;
        try {
            languacecode = languagecodeMgr.findByIsocode(isocode);
        } catch (NoResultException e) {
            throw new Exception(this.localizationres.getString("error.additionmgr.getlng.nol"));
        } catch (Exception e) {
            throw new Exception(this.localizationres.getString("error.additionmgr.getlng.err"));
        }
        return languacecode;
    }

    private Itemclass getItemclass(String uriname) throws Exception {
        ItemclassMgr itemclassMgr = new ItemclassMgr(em);
        Itemclass itemClass = null;
        try {
            itemClass = itemclassMgr.getItemclassByUriname(uriname);
        } catch (NoResultException e) {
            throw new Exception(this.localizationres.getString("error.additionmgr.itmc.noritmc").replace("{0}", uriname));
        }
        if (itemClass.getRegister() == null) {
            throw new Exception(this.localizationres.getString("error.additionmgr.itmc.norr").replace("{0}", uriname));
        }
        return itemClass;
    }

    private Status getStatus(String uriname) throws Exception {
        StatusMgr statusMgr = new StatusMgr(em);
        Status status = null;
        try {
            status = statusMgr.getStatusByUriname(uriname);
        } catch (NoResultException e) {
            throw new Exception(this.localizationres.getString("error.additionmgr.st.norst").replace("{0}", uriname));
        }
        return status;
    }

    private void storeAdditionChangelog(Item item, Addition addition) throws Exception {
        ChangelogMgr changelogMgr = new ChangelogMgr(this.em);

        Changelog changelog = new Changelog();
        String changelogUuid = Changelog.createUuid(item, System.currentTimeMillis());

        changelog.setUuid(changelogUuid);
        changelog.setAction(Constants.ACTION_ADDITION);
        changelog.setComment(addition.getComment());
        changelog.setDataprocedure(this.dataprocedure);
        changelog.setDatecreation(new Date());
        changelog.setItem(item);
        //changelog.setLanguagecode(languagecode);

        changelogMgr.insertChangelog(changelog);
    }

    private void createNewParentRelation(Item item, Item parent) throws Exception {
        ItemparentMgr itemParentMgr = new ItemparentMgr(this.em);

        String itemparentUuid = Itemparent.createUuid(item.getUuid(), parent.getUuid());

        Itemparent itemparent = new Itemparent();

        itemparent.setUuid(itemparentUuid);
        itemparent.setItem(item);
        itemparent.setParent(parent);
        itemparent.setDatecreation(new Date());

        itemParentMgr.insertItemparent(itemparent);
    }

    private void createNewCollectionRelation(Item item, Item collection) throws Exception {
        ItemcollectionMgr itemCollectionMgr = new ItemcollectionMgr(this.em);

        String itemcollectionUuid = Itemcollection.createUuid(item.getUuid(), collection.getUuid());

        Itemcollection itemcollection = new Itemcollection();

        itemcollection.setUuid(itemcollectionUuid);
        itemcollection.setItem(item);
        itemcollection.setCollection(collection);
        itemcollection.setDatecreation(new Date());

        itemCollectionMgr.insertItemcollection(itemcollection);
    }

    /**
     * **Analyzer***
     */
    public void analyzeAdditions() {
        log.info("# Starting to analyze additions.");

        //Starting the analyzer for each addition
        for (String key : this.additions.keySet()) {
            Addition addition = null;
            try {
                addition = this.additions.get(key);
                this.analyzeAddition(addition);

            } catch (ErrorReport e) {
                //If an error occurs during the analyzer operation  
                //save a error message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_ANALYZEADDITION, "Item: " + ((addition != null) ? addition.getItemClass() + "/" + addition.getLocalId() + ((addition.getCollectionLocalId() != null && addition.getCollectionLocalId().length() > 0) ? "/" + addition.getCollectionLocalId() : "") + "/" + addition.getLanguage() : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                log.error("@@ Item: " + ((addition != null) ? addition.getItemClass() + "/" + addition.getLocalId() + ((addition.getCollectionLocalId() != null && addition.getCollectionLocalId().length() > 0) ? "/" + addition.getCollectionLocalId() : "") + "/" + addition.getLanguage() : key) + ". Detail: " + e.getMessage());
            } catch (WarningReport e) {
                //If an warningr occurs during the analyzer operation  
                //save a warning message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_ANALYZEADDITION, "Item: " + ((addition != null) ? addition.getItemClass() + "/" + addition.getLocalId() + ((addition.getCollectionLocalId() != null && addition.getCollectionLocalId().length() > 0) ? "/" + addition.getCollectionLocalId() : "") + "/" + addition.getLanguage() : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.warn("@@ Item: " + ((addition != null) ? addition.getItemClass() + "/" + addition.getLocalId() + ((addition.getCollectionLocalId() != null && addition.getCollectionLocalId().length() > 0) ? "/" + addition.getCollectionLocalId() : "") + "/" + addition.getLanguage() : key) + ". Detail: " + e.getMessage());
            } catch (Exception e) {
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_ADDITIONMGR, Constants.REPORT_ACTION_ANALYZEADDITION, "Item: " + ((addition != null) ? addition.getItemClass() + "/" + addition.getLocalId() + ((addition.getCollectionLocalId() != null && addition.getCollectionLocalId().length() > 0) ? "/" + addition.getCollectionLocalId() : "") + "/" + addition.getLanguage() : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                log.error("@@ Item: " + ((addition != null) ? addition.getItemClass() + "/" + addition.getLocalId() + ((addition.getCollectionLocalId() != null && addition.getCollectionLocalId().length() > 0) ? "/" + addition.getCollectionLocalId() : "") + "/" + addition.getLanguage() : key) + ". Detail: " + e.getMessage());
            }
        }

        log.info("# Additions analyzed.");

    }

    private void analyzeAddition(Addition addition) throws ErrorReport, WarningReport {

        ItemMgr itemMgr = new ItemMgr(this.em);
        //Getting the itemclass object
        ItemclassMgr itemclassMgr = new ItemclassMgr(this.em);
        Itemclass itmc = null;
        try {
            itmc = itemclassMgr.getItemclassByUriname(addition.getItemClass());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.noitmc").replace("{0}", addition.getLocalId()));
        }

        /**
         * * Starting checks **
         */
        //Checking if the element is already in the database / duplicated in the addition file
        try {
            this.checkUniqueness(addition, itemMgr);
        } catch (ErrorReport e) {
            throw e;
        } catch (Exception e) {
            throw new ErrorReport("Check uniqueness - Generic error: " + e.getMessage());
        }

        //Checking if an element in the data file element has all the language specified;
        if (!addition.isExternalItem()) {
            try {
                this.checkItemLanguages(addition, itemMgr);
            } catch (ErrorReport e) {
                throw e;
            } catch (Exception e) {
                throw new ErrorReport("Check item language - Generic error: " + e.getMessage());
            }
        }

        //Checking parent
        if (!addition.isExternalItem()) {
            try {
                this.checkParent(addition, itemMgr);
            } catch (ErrorReport e) {
                throw e;
            } catch (Exception e) {
                throw new ErrorReport("Check parent - Generic error: " + e.getMessage());
            }
        }

        //Checking if the item has a parent itemclass (if so, it needs the collection collection)
        String parentItemClass = "";
        try {
            parentItemClass = this.checkItemclass(itmc, addition);
        } catch (ErrorReport e) {
            throw e;
        } catch (Exception e) {
            throw new ErrorReport("Check parent itemclass - Generic error: " + e.getMessage());
        }

        //Checking collection
        try {
            this.checkCollection(addition, itemMgr, parentItemClass, itmc);
        } catch (ErrorReport e) {
            throw e;
        } catch (Exception e) {
            throw new ErrorReport("Check collection - Generic error: " + e.getMessage());
        }
        //Checking custom attribute
        if (!addition.isExternalItem()) {
            try {
                this.checkCustomAttribute(addition, itemMgr);
            } catch (ErrorReport e) {
                throw e;
            } catch (Exception e) {
                throw new ErrorReport("Check custom attribute - Generic error: " + e.getMessage());
            }
        }

    }

    private void checkUniqueness(Addition addition, ItemMgr itemMgr) throws ErrorReport {
        try {
            //Checking if the element is already in the database
            String itemUuid = Item.createUuid(addition.getLocalId(), this.latestVersion, addition.getCollectionLocalId(), addition.getItemClass());
            Item item = itemMgr.getItemByUuid(itemUuid);

            //Checking if this is a "language attition"
            String operatingLanguage = this.properties.getProperty(Constants.KEY_OPERATING_LANGUAGES);
            if (!addition.getLanguage().equals(operatingLanguage)) {
                //Check if this item with this language is not already in the database
                LocalizationMgr localizationMgr = new LocalizationMgr(this.em);
                LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(this.em);
                Languagecode languagecode = languagecodeMgr.findByIsocode(addition.getLanguage());
                localizationMgr.findByItemAndLanguagecode(item, languagecode);
                throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.unique"));
            } else {
                throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.unique"));
            }
        } catch (NoResultException e) {
            //If there are no duplicates, continue
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.uniqueerr").replace("{0}", e.getMessage()));
        }
    }

    private void checkItemLanguages(Addition addition, ItemMgr itemMgr) throws ErrorReport, WarningReport {
        //The operating language of an item shall be always available;
        String operatingLanguage = this.properties.getProperty(Constants.KEY_OPERATING_LANGUAGES);

        String additionItemOperatingLanguageKey = Addition.getAdditionHashKey(addition.getLocalId(), addition.getCollectionLocalId(), addition.getItemClass(), addition.getLanguage());
        String checkItemOperatingLanguageKey = Addition.getAdditionHashKey(addition.getLocalId(), addition.getCollectionLocalId(), addition.getItemClass(), operatingLanguage);

        //If the current imported item's language is not the operating language, 
        //check if the operating language is available in the current import file or
        //already in the database
        if (!additionItemOperatingLanguageKey.equals(checkItemOperatingLanguageKey) && !this.additions.containsKey(checkItemOperatingLanguageKey)) {
            String checkItemUuid = null;
            try {
                checkItemUuid = Item.createUuid(addition.getLocalId(), latestVersion, addition.getCollectionLocalId(), addition.getItemClass());
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.opl"));
            }
            try {
                Item item = itemMgr.getItemByUuid(checkItemUuid);
            } catch (NoResultException e) {
                throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.oplna"));
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.opl"));
            }

        }

        //Checking if the current item's language is in the supported language list
        String supportedLanguage = this.properties.getProperty(Constants.KEY_DATA_LANGUAGES);
        String requiredLanguages[] = (supportedLanguage != null) ? supportedLanguage.split("-") : null;
        if (requiredLanguages != null) {
            boolean lngFound = false;
            for (String tmpLng : requiredLanguages) {
                if (addition.getLanguage().equals(tmpLng)) {
                    lngFound = true;
                    break;
                }
            }
            if (!lngFound) {
                throw new WarningReport(this.localizationres.getString("error.additionmgr.anad.slnota"));
            }
        } else {
            throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.slnotaerr"));
        }
    }

    private void checkParent(Addition addition, ItemMgr itemMgr) throws ErrorReport, WarningReport {
        //Checking parent (if specified in the data file)
        if (addition.getParentLocalId() != null && addition.getParentLocalId().length() > 0) {

            String[] parents = addition.getParentLocalId().split(Constants.MULTIPLEPARENT_SEPARATOR);
            for (String parentItem : parents) {
                try {
                    //Checking if the parent is available
                    String itemUuid = Item.createUuid(parentItem, this.latestVersion, addition.getCollectionLocalId(), addition.getItemClass());
                    Item item = itemMgr.getItemByUuid(itemUuid);

                    //Checking if the parent has a valid status
                    if (!item.getStatus().getUuid().equals(Constants.ITEMSTATUS_VALID)) {
                        String tmp = this.localizationres.getString("error.additionmgr.anad.parentnov").replace("{0}", item.getUriname()).replace("{1}", item.getStatus().getUriname());
                        throw new WarningReport(tmp);
                    }

                } catch (NoResultException e) {
                    String tmpKey = null;
                    try {
                        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
                        tmpKey = Addition.getAdditionHashKey(parentItem, addition.getCollectionLocalId(), addition.getItemClass(), languagecodeMgr.getMasterLanguagecode().getIsocode());
                    } catch (Exception ee) {
                        throw new ErrorReport(this.localizationres.getString("error.additionmgr.no.masterlanguage").replace("{0}", ee.getMessage()));
                    }
                    if (this.additions.containsKey(tmpKey)) {
                        Addition tmpAddition = this.additions.get(tmpKey);
                        if (tmpAddition.getStatus() != null && !tmpAddition.getStatus().equals(Constants.ITEMSTATUS_VALID_TEXT)) {
                            String tmp = this.localizationres.getString("error.additionmgr.anad.parentnov").replace("{0}", parentItem).replace("{1}", tmpAddition.getStatus());
                            throw new WarningReport(tmp);
                        }

                    } else {
                        throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.parentnoav"));
                    }
                } catch (ErrorReport e) {
                    throw new ErrorReport(e.getMessage());
                } catch (WarningReport e) {
                    throw new WarningReport(e.getMessage());
                } catch (Exception e) {
                    throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.parenterr").replace("{0}", e.getMessage()));
                }
            }
        }
    }

    private String checkItemclass(Itemclass itmc, Addition addition) throws ErrorReport {
        //Checking if the item has a parent itemclass (if so, it needs the collection collection)
        String parentItemClass = (itmc != null && itmc.getParent() != null) ? itmc.getParent().getUriname() : "";
        if (parentItemClass != null && parentItemClass.length() > 0 && (addition.getCollectionLocalId() == null || addition.getCollectionLocalId().length() <= 0)) {
            throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.itmce"));
        }
        return parentItemClass;
    }

    private void checkCollection(Addition addition, ItemMgr itemMgr, String parentItemClass, Itemclass itmc) throws ErrorReport, WarningReport {
        //Checking collection
        if (addition.getCollectionLocalId() != null && addition.getCollectionLocalId().length() > 0) {
            try {
                String itemUuid = Item.createUuid(addition.getCollectionLocalId(), this.latestVersion, null, itmc.getParent().getUriname());
                Item item = itemMgr.getItemByUuid(itemUuid);

                //Checking if the collection has a valid status
                if (!item.getStatus().getUuid().equals(Constants.ITEMSTATUS_VALID)) {
                    String tmp = this.localizationres.getString("error.additionmgr.anad.cnov").replace("{0}", item.getUriname()).replace("{1}", item.getStatus().getUriname());
                    throw new WarningReport(tmp);
                }
            } catch (NoResultException e) {
                try {
                    LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);

                    if (!this.additions.containsKey(Addition.getAdditionHashKey(addition.getCollectionLocalId(), "", parentItemClass, languagecodeMgr.getMasterLanguagecode().getIsocode()))) {
                        throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.cnav").replace("{0}", addition.getCollectionLocalId()));
                    }
                    String tmpKey = Addition.getAdditionHashKey(addition.getCollectionLocalId(), "", parentItemClass, languagecodeMgr.getMasterLanguagecode().getIsocode());
                    if (this.additions.containsKey(tmpKey)) {
                        Addition tmpAddition = this.additions.get(tmpKey);
                        if (tmpAddition.getStatus() != null && !tmpAddition.getStatus().equals(Constants.ITEMSTATUS_VALID_TEXT)) {
                            String tmp = this.localizationres.getString("error.additionmgr.anad.cnov").replace("{0}", addition.getCollectionLocalId()).replace("{1}", tmpAddition.getStatus());
                            throw new WarningReport(tmp);
                        }
                    } else {
                        throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.cnav1"));
                    }
                } catch (ErrorReport er) {
                    throw new ErrorReport(er.getMessage());
                } catch (WarningReport ew) {
                    throw new WarningReport(ew.getMessage());
                } catch (Exception ee) {
                    throw new ErrorReport(this.localizationres.getString("error.additionmgr.no.masterlanguage").replace("{0}", ee.getMessage()));
                }
            } catch (ErrorReport e) {
                throw new ErrorReport(e.getMessage());
            } catch (WarningReport e) {
                throw new WarningReport(e.getMessage());
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.ccerr").replace("{0}", e.getMessage()));
            }
        }
    }

    private void checkCustomAttribute(Addition addition, ItemMgr itemMgr) throws ErrorReport, WarningReport {
        //Checking custom attribute
        ArrayList<CustomattributeDescriptor> customAttributeDescriptors = addition.getCustomAttributeDescriptors();

        for (CustomattributeDescriptor customAttributeDescriptor : customAttributeDescriptors) {

            if (customAttributeDescriptor.isForeignkey() && customAttributeDescriptor.getValue() != null && customAttributeDescriptor.getValue().length() > 0) {
                //Getting the external key element

                //Check also the multivalued custom attribute
                String tmpvals = customAttributeDescriptor.getValue();
                String[] tmpval = tmpvals.split(",");

                try {
                    for (String val : tmpval) {
                        // Check if the reference is to an external value
                        if (val.contains(properties.getProperty("data.externalitems.check"))) {

                            // Check if the URL resolves
                            /*
                            log.info("# Checking URL: " + val);
                            URL url = new URL(val);
                            HttpURLConnection huc = (HttpURLConnection) url.openConnection();
                            huc.setInstanceFollowRedirects(false);
                            huc.setFollowRedirects(false);
                            huc.connect();
                            if (huc.getResponseCode() != HttpURLConnection.HTTP_OK) {
                                log.info("# URL error: " + val + " - Response code: " + huc.getResponseCode());
                                throw new ErrorReport(this.localizationres.getString("error.additionmgr.externalca.notvalid") + val);
                            }
                            log.info("# URL check finished: " + val);
                            */
                        } else {
                            // If the reference is internal, check if the referenced element is available 
                            String itemUuid = Item.createUuid(val, this.latestVersion, null, customAttributeDescriptor.getName());
                            Item item = itemMgr.getItemByUuid(itemUuid);

                            //Checking if the custom attribute pointed element has a valid status
                            if (!item.getStatus().getUuid().equals(Constants.ITEMSTATUS_VALID)) {
                                String tmp = this.localizationres.getString("error.additionmgr.anad.cca.caelnv").replace("{0}", addition.getLocalId()).replace("{1}", customAttributeDescriptor.getName()).replace("{2}", val);
                                throw new WarningReport(tmp);
                            }
                        }
                    }
                } catch (NoResultException e) {
                    try {
                        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
                        //If the pointed element is not available in the db, the system search in the addition 
                        for (String val : tmpval) {
                            String tmpKey = Addition.getAdditionHashKey(val, "", customAttributeDescriptor.getName(), languagecodeMgr.getMasterLanguagecode().getIsocode());
                            if (this.additions.containsKey(tmpKey)) {
                                Addition tmpAddition = this.additions.get(tmpKey);
                                if (tmpAddition.getStatus() != null && !tmpAddition.getStatus().equals(Constants.ITEMSTATUS_VALID_TEXT)) {
                                    String tmp = this.localizationres.getString("error.additionmgr.anad.cca.caelnv1").replace("{0}", addition.getLocalId()).replace("{1}", customAttributeDescriptor.getName()).replace("{2}", val);
                                    throw new ErrorReport(tmp);
                                }
                            } else {
                                String tmp = this.localizationres.getString("error.additionmgr.anad.cca.caelnv2").replace("{0}", customAttributeDescriptor.getName()).replace("{1}", val);
                                throw new ErrorReport(tmp);
                            }
                        }
                    } catch (ErrorReport er) {
                        throw new ErrorReport(er.getMessage());
                    } catch (WarningReport ew) {
                        throw new WarningReport(ew.getMessage());
                    } catch (Exception ee) {
                        throw new ErrorReport(this.localizationres.getString("error.additionmgr.no.masterlanguage").replace("{0}", ee.getMessage()));
                    }
                } catch (ErrorReport e) {
                    throw new ErrorReport(e.getMessage());
                } catch (WarningReport e) {
                    throw new WarningReport(e.getMessage());
                } catch (Exception e) {
                    throw new ErrorReport(this.localizationres.getString("error.additionmgr.anad.ccerr").replace("{0}", e.getMessage()));
                }
            }
        }
    }
}
