/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: --/--/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller.ror;

import eu.europa.ec.re3gistrycommon.managers.ItemMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemclassMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemcollectionMgr;
import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrycommon.utils.StringUtils;
import eu.europa.ec.re3gistrystaticizer.StaticizeFiles;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.ItemRORStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegisterRORStandard;
import eu.europa.ec.re3gistrystaticizer.util.Constants;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DefinitionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DescriptionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.LabelAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.list.ItemRORStandardList;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.persistence.EntityManager;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.sf.saxon.TransformerFactoryImpl;

public class RegisterROR {

    private final EntityManager em;

    private final String folderSeparator;
    private final String folderStandardStaticizer;

    private final String folderCustomStaticizer;
    private final StaticizeFiles staticizeFiles;

    private final List<Languagecode> allLanguagecode;
    private final Languagecode masterLanguage;

    private static org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(Constants.MODULE_NAME);
    private final ReportMgr reportMgr;

    /**
     *
     * @param staticizeFiles
     * @param em
     * @throws java.lang.Exception
     */
    public RegisterROR(StaticizeFiles staticizeFiles, EntityManager em) throws Exception {
        this.staticizeFiles = staticizeFiles;
        this.folderSeparator = staticizeFiles.getFolderSeparator();
        this.folderStandardStaticizer = staticizeFiles.getCreateStandardFiles().getFolderStandardStaticizer();
        this.folderCustomStaticizer = staticizeFiles.getCreateStandardFiles().getFolderCustomStaticizer();

        this.reportMgr = staticizeFiles.getReportMgr();
        this.em = em;

        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
        allLanguagecode = languagecodeMgr.findAll();

        masterLanguage = languagecodeMgr.getMasterLanguagecode();
    }

    public List<File> addRegisterFiles(LinkedHashMap<Itemclass, List<Languagecode>> registerToStaticize, LinkedHashMap<Item, List<Languagecode>> itemsTostaticize) throws Exception {
        List<File> list = new ArrayList<File>();
        if (staticizeFiles.getCreateStandardFiles().getEXTENSIONS().length != 0 && !staticizeFiles.getCreateStandardFiles().getEXTENSIONS()[0].equals("")) {
            try {
                createRORDescriptorForFlatRegisters(registerToStaticize, list);
                createRORDescriptorForHierarchicalRegisters(itemsTostaticize, list);

            } catch (Exception ex) {
                String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                staticizeFiles.getReportMgr().setMailSubject(subject);
                staticizeFiles.getReportMgr().setMailBody(body);
                staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("Error in writing Standard && Custom Item Files for the Registry - RegisterROR", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.info("=================================Error in writing Standard && Custom Item Files for the Registry - RegisterROR=============================================" + ex.getMessage());
                throw new Exception("Has been encountered a problem while writing the custom item files  for the Registry. ");
            }
        }
        return list;
    }

    private void createRORDescriptorForHierarchicalRegisters(LinkedHashMap<Item, List<Languagecode>> itemsTostaticize, List<File> list) throws Exception {
        ItemclassMgr itemclassMgr = new ItemclassMgr(em);
        for (Map.Entry<Item, List<Languagecode>> entry : itemsTostaticize.entrySet()) {
            Item itemToAddToROR = entry.getKey();
            /**
             * if the item is part of a hierarchical register and is first level
             * of item class than create the ror file
             */
            Itemclass itemclassItem = itemToAddToROR.getItemclass();
            final Register register = itemToAddToROR.getItemclass().getRegister();
            Itemclass itemclassFirstLevel = itemclassMgr.getItemClassFirstLevelIfRegisterHierarchical(register);

            if (itemclassFirstLevel != null && (itemclassItem.equals(itemclassFirstLevel))) {
                /**
                 * create item.ror for the flat register
                 */
                RegisterRORStandard registerRORStandard = new RegisterRORStandard();
                final String uriRegister = register.getBaseuri() + "/" + itemToAddToROR.getItemclass().getRegister().getUriname();
                final String registry = register.getBaseuri() + "/" + itemToAddToROR.getItemclass().getRegister().getRegistry().getUriname();
                registerRORStandard.setUri(uriRegister + "/" + itemToAddToROR.getUriname());
                registerRORStandard.setUriname(itemToAddToROR.getUriname());
                registerRORStandard.setRegistry(registry);

                LocalizationMgr localizationMgr = new LocalizationMgr(em);
                Localization itemToAddToRORLocalization = localizationMgr.findByItemAndLanguagecode(itemToAddToROR, masterLanguage);
                String registerLanguageLocalization = itemToAddToRORLocalization.getLanguage().getIsocode();

                LabelAttribute registryLabelAttribute = new LabelAttribute();
                registryLabelAttribute.setLanguage(registerLanguageLocalization);
                registryLabelAttribute.setValue(itemToAddToRORLocalization.getLabel());
                registerRORStandard.setLabel(registryLabelAttribute);

                StringUtils stringUtils = new StringUtils();

                if (itemToAddToRORLocalization.getDefinition() != null) {
                    DefinitionAttribute registryDefinitionAttribute = new DefinitionAttribute();
                    registryDefinitionAttribute.setLanguage(registerLanguageLocalization);

                    String definition = stringUtils.replaceSpecialCharacters(itemToAddToRORLocalization.getDefinition());
                    registryDefinitionAttribute.setValue(definition);
                    registerRORStandard.setDefinition(registryDefinitionAttribute);
                }

                if (itemToAddToRORLocalization.getDescription() != null) {
                    DescriptionAttribute registryDescriptionAttribute = new DescriptionAttribute();
                    registryDescriptionAttribute.setLanguage(registerLanguageLocalization);

                    String description = stringUtils.replaceSpecialCharacters(itemToAddToRORLocalization.getDescription());
                    registryDescriptionAttribute.setValue(description);
                    registerRORStandard.setDescription(registryDescriptionAttribute);
                }

                /**
                 * get item children
                 */
                ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(em);
                List<Itemcollection> itemList = itemcollectionMgr.findByCollection(itemToAddToROR);

                if (itemList != null) {
                    ItemRORStandardList itemRORStandardList = new ItemRORStandardList();
                    ArrayList<ItemRORStandard> itemRORStandardArray = new ArrayList<ItemRORStandard>();
                    for (Itemcollection itemcollection : itemList) {
                        Item item = itemcollection.getItem();
                        ItemRORStandard itemRORStandard = new ItemRORStandard();

                        if (item.getUriname().contains("://")) {
                            itemRORStandard.setRegister(item.getItemclass().getRegister().getBaseuri() + "/" + item.getItemclass().getRegister().getUriname());
                            itemRORStandard.setUri(item.getUriname());
                        } else {

                            final String inScheme = item.getItemclass().getRegister().getBaseuri() + "/" + item.getItemclass().getRegister().getUriname() + "/" + itemToAddToROR.getUriname();
                            final String uri = inScheme + "/" + item.getUriname();
                            itemRORStandard.setUri(uri);
                            itemRORStandard.setRegister(inScheme);

                            if (item.getUriname() != null) {
                                itemRORStandard.setUriname(item.getUriname());
                                itemRORStandard.setStatus(item.getStatus().getBaseuri() + "/" + item.getStatus().getUriname());

                                /**
                                 * label, definition
                                 */
                                Localization itemRORLocalization = localizationMgr.findByItemAndLanguagecode(item, masterLanguage);
                                String itemLanguageLocalization = itemRORLocalization.getLanguage().getIsocode();

                                LabelAttribute itemLabelAttribute = new LabelAttribute();
                                itemLabelAttribute.setLanguage(itemLanguageLocalization);
                                itemLabelAttribute.setValue(itemRORLocalization.getLabel());
                                itemRORStandard.setLabel(itemLabelAttribute);

                                if (itemRORLocalization.getDefinition() != null) {
                                    DefinitionAttribute registryDefinitionAttribute = new DefinitionAttribute();
                                    registryDefinitionAttribute.setLanguage(itemLanguageLocalization);

                                    String definition = stringUtils.replaceSpecialCharacters(itemRORLocalization.getDefinition());
                                    registryDefinitionAttribute.setValue(definition);
                                    itemRORStandard.setDefinition(registryDefinitionAttribute);
                                }

                                if (itemRORLocalization.getDescription() != null) {
                                    DescriptionAttribute registryDescriptionAttribute = new DescriptionAttribute();
                                    registryDescriptionAttribute.setLanguage(itemLanguageLocalization);

                                    String description = stringUtils.replaceSpecialCharacters(itemRORLocalization.getDescription());
                                    registryDescriptionAttribute.setValue(description);
                                    itemRORStandard.setDescription(registryDescriptionAttribute);
                                }
                            }
                        }

                        itemRORStandardArray.add(itemRORStandard);
                    }
                    itemRORStandardList.setItemRORStandard(itemRORStandardArray);
                    registerRORStandard.setItemRORStandardList(itemRORStandardList);
                }
                list.add(createXMLHierarchicalRegisterRORFile(registerRORStandard, itemToAddToROR, masterLanguage.getIsocode()));
            }
        }
    }

    private void createRORDescriptorForFlatRegisters(LinkedHashMap<Itemclass, List<Languagecode>> registerToStaticize, List<File> list) throws Exception {
        ItemclassMgr itemclassMgr = new ItemclassMgr(em);
        ItemMgr itemMgr = new ItemMgr(em);
        for (Map.Entry<Itemclass, List<Languagecode>> entry : registerToStaticize.entrySet()) {
            Itemclass itemclass = entry.getKey();
            final Register register = itemclass.getRegister();
            Itemclass itemclassFirstLevel = itemclassMgr.getItemClassFirstLevelIfRegisterHierarchical(register);
            if (itemclassFirstLevel == null) {
                /**
                 * create item.ror for the flat register
                 */
                RegisterRORStandard registerRORStandard = new RegisterRORStandard();
                registerRORStandard.setUri(register.getBaseuri() + "/" + register.getUriname());
                registerRORStandard.setUriname(register.getUriname());
                registerRORStandard.setRegistry(register.getRegistry().getBaseuri() + "/" + register.getRegistry().getUriname());

                LocalizationMgr localizationMgr = new LocalizationMgr(em);
                Localization registryRORLocalization = localizationMgr.findByRegisterAndLanguagecode(register, masterLanguage);
                String registerLanguageLocalization = registryRORLocalization.getLanguage().getIsocode();

                LabelAttribute registryLabelAttribute = new LabelAttribute();
                registryLabelAttribute.setLanguage(registerLanguageLocalization);
                registryLabelAttribute.setValue(registryRORLocalization.getLabel());
                registerRORStandard.setLabel(registryLabelAttribute);

                StringUtils stringUtils = new StringUtils();

                if (registryRORLocalization.getDefinition() != null) {
                    DefinitionAttribute registryDefinitionAttribute = new DefinitionAttribute();
                    registryDefinitionAttribute.setLanguage(registerLanguageLocalization);

                    String definition = stringUtils.replaceSpecialCharacters(registryRORLocalization.getDefinition());
                    registryDefinitionAttribute.setValue(definition);
                    registerRORStandard.setDefinition(registryDefinitionAttribute);
                }

                if (registryRORLocalization.getDescription() != null) {
                    DescriptionAttribute registryDescriptionAttribute = new DescriptionAttribute();
                    registryDescriptionAttribute.setLanguage(registerLanguageLocalization);

                    String description = stringUtils.replaceSpecialCharacters(registryRORLocalization.getDescription());
                    registryDescriptionAttribute.setValue(description);
                    registerRORStandard.setDescription(registryDescriptionAttribute);
                }

                List<Item> itemList = itemMgr.findItemsByItemclass(itemclass);
                ItemRORStandardList itemRORStandardList = new ItemRORStandardList();
                ArrayList<ItemRORStandard> itemRORStandardArray = new ArrayList<ItemRORStandard>();
                for (Item item : itemList) {

                    ItemRORStandard itemRORStandard = new ItemRORStandard();

                    if (item.getUriname().contains("://")) {
                        itemRORStandard.setRegister(item.getItemclass().getRegister().getBaseuri() + "/" + item.getItemclass().getRegister().getUriname());
                        itemRORStandard.setUri(item.getUriname());
                    } else {
                        itemRORStandard.setRegister(item.getItemclass().getRegister().getBaseuri() + "/" + item.getItemclass().getRegister().getUriname());
                        itemRORStandard.setUriname(item.getUriname());

                        final String uri = item.getItemclass().getRegister().getBaseuri() + "/" + item.getItemclass().getRegister().getUriname() + "/" + item.getUriname();

                        itemRORStandard.setUri(uri);
                        itemRORStandard.setStatus(item.getStatus().getBaseuri() + "/" + item.getStatus().getUriname());

                        /**
                         * label, definition
                         */
                        Localization itemRORLocalization = localizationMgr.findByItemAndLanguagecode(item, masterLanguage);
                        String itemLanguageLocalization = itemRORLocalization.getLanguage().getIsocode();

                        LabelAttribute itemLabelAttribute = new LabelAttribute();
                        itemLabelAttribute.setLanguage(itemLanguageLocalization);
                        itemLabelAttribute.setValue(itemRORLocalization.getLabel());
                        itemRORStandard.setLabel(itemLabelAttribute);

                        if (itemRORLocalization.getDefinition() != null) {
                            DefinitionAttribute registryDefinitionAttribute = new DefinitionAttribute();
                            registryDefinitionAttribute.setLanguage(itemLanguageLocalization);

                            String definition = stringUtils.replaceSpecialCharacters(itemRORLocalization.getDefinition());
                            registryDefinitionAttribute.setValue(definition);
                            itemRORStandard.setDefinition(registryDefinitionAttribute);
                        }

                        if (itemRORLocalization.getDescription() != null) {
                            DescriptionAttribute registryDescriptionAttribute = new DescriptionAttribute();
                            registryDescriptionAttribute.setLanguage(itemLanguageLocalization);

                            String description = stringUtils.replaceSpecialCharacters(itemRORLocalization.getDescription());
                            registryDescriptionAttribute.setValue(description);
                            itemRORStandard.setDescription(registryDescriptionAttribute);
                        }
                    }
                    itemRORStandardArray.add(itemRORStandard);
                }
                itemRORStandardList.setItemRORStandard(itemRORStandardArray);
                registerRORStandard.setItemRORStandardList(itemRORStandardList);

                list.add(createXMLFlatRegisterRORFile(registerRORStandard, masterLanguage.getIsocode()));
            }
        }
    }

    private File createXMLFlatRegisterRORFile(RegisterRORStandard registerRORStandard, String isocode) throws Exception {
        String registryStandardFolderPath = folderStandardStaticizer;
        String registryCustomFolderPath = folderCustomStaticizer;
        File registryStandardFolder = new File(registryStandardFolderPath);
        File registryCustomFolder = new File(registryCustomFolderPath);

        if (!registryStandardFolder.exists()) {
            registryStandardFolder.mkdirs();
        }
        if (!registryCustomFolder.exists()) {
            registryCustomFolder.mkdirs();
        }

        String register = registerRORStandard.getUriname();
        String registryFilePath = registryStandardFolder.getAbsolutePath() + folderSeparator + register + "/" + register + /*".descriptor."  + isocode +*/ ".xml";
        File registryFile = new File(registryFilePath);

        try {
            /**
             * STANDARD XML
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(RegisterRORStandard.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            marshallerclassLanguage.marshal(registerRORStandard, registryFile);
            log.info("Create with success: " + registryFile);

            String ext = "ror";
            try {
                String registerXSL = staticizeFiles.getStaticizerXSLFolder() + "/" + ext + "/" + /*register +*/ "register." + ext + ".xsl";
                String finalOutputPath = registryFile.getAbsolutePath().replace("standard", "custom").replace("xml", ext);

                File finalOutFile = new File(finalOutputPath);
                finalOutFile.createNewFile();
                File transformerFile = new File(registerXSL);

                TransformerFactoryImpl factory = new net.sf.saxon.TransformerFactoryImpl();
                Transformer transformer = factory.newTransformer(new StreamSource(transformerFile));
                transformer.transform(new StreamSource(registryFile), new StreamResult(finalOutFile));
                log.info("Create with success: " + finalOutFile);
            } catch (IOException ex) {
                String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                staticizeFiles.getReportMgr().setMailSubject(subject);
                staticizeFiles.getReportMgr().setMailBody(body);
                staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
                throw new Exception("Has been encountered a problem while writing the custom item files. ");
            } catch (TransformerException ex) {
                String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                staticizeFiles.getReportMgr().setMailSubject(subject);
                staticizeFiles.getReportMgr().setMailBody(body);
                staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);
                
                this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);
                
                log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
                throw new Exception("Has been encountered a problem while writing the custom item files. ");
            }

        } catch (JAXBException ex) {
            String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
            String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
            staticizeFiles.getReportMgr().setMailSubject(subject);
            staticizeFiles.getReportMgr().setMailBody(body);
            staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom item files. ");
        }

        return registryCustomFolder;
    }

    private File createXMLHierarchicalRegisterRORFile(RegisterRORStandard itemRORStandard, Item itemToAddToROR, String isocode) throws Exception {
        String registryStandardFolderPath = folderStandardStaticizer;
        String registryCustomFolderPath = folderCustomStaticizer;
        File registryStandardFolder = new File(registryStandardFolderPath);
        File registryCustomFolder = new File(registryCustomFolderPath);

        if (!registryStandardFolder.exists()) {
            registryStandardFolder.mkdirs();
        }
        if (!registryCustomFolder.exists()) {
            registryCustomFolder.mkdirs();
        }

        String register = itemToAddToROR.getItemclass().getRegister().getUriname() + "/" + itemToAddToROR.getUriname();
        String itemFilePath = registryStandardFolder.getAbsolutePath() + folderSeparator + register + "/" + itemToAddToROR.getUriname() + /*".descriptor."  + isocode +*/ ".xml";
        File itemFile = new File(itemFilePath);

        try {
            /**
             * STANDARD XML
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(RegisterRORStandard.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            marshallerclassLanguage.marshal(itemRORStandard, itemFile);
            log.info("Create with success: " + itemFile);

            String ext = "ror";
            try {
                String registerXSL = staticizeFiles.getStaticizerXSLFolder() + "/" + ext + "/" + /*register +*/ "register." + ext + ".xsl";
                String finalOutputPath = itemFile.getAbsolutePath().replace("standard", "custom").replace("xml", ext);

                File finalOutFile = new File(finalOutputPath);
                finalOutFile.createNewFile();
                File transformerFile = new File(registerXSL);

                TransformerFactoryImpl factory = new net.sf.saxon.TransformerFactoryImpl();
                Transformer transformer = factory.newTransformer(new StreamSource(transformerFile));
                transformer.transform(new StreamSource(itemFile), new StreamResult(finalOutFile));
                log.info("Create with success: " + finalOutFile);
            } catch (Exception ex) {
                String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                staticizeFiles.getReportMgr().setMailSubject(subject);
                staticizeFiles.getReportMgr().setMailBody(body);
                staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
                throw new Exception("Has been encountered a problem while writing the custom item files. ");
            }

        } catch (JAXBException ex) {
            String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
            String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
            staticizeFiles.getReportMgr().setMailSubject(subject);
            staticizeFiles.getReportMgr().setMailBody(body);
            staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom item files. ");
        }

        return registryCustomFolder;
    }

}
